#
#

class Constants:

    ANONYMOUS = 'ANONYMOUS'
    EVENT_DEFAULT = 'tyo-mq-mt-default'
    SYSTEM = 'TYO-MQ-SYSTEM'
    ALL_PUBLISHERS = 'TYO-MQ-ALL'

    METHOD_BROADCAST = 'broadcast'

    # SERVER DEFAULTS
    DEFAULT_PORT = 17352